import configparser
import os
import stat
import subprocess
import sys
import unittest
from pathlib import Path
from apsimNGpy.core.pythonet_config import is_file_format_modified
import logging
import smtplib
from email.mime.text import MIMEText
from apsimNGpy.core.config import get_apsim_bin_path, apsim_version
from datetime import datetime
from email import encoders

from pathlib import Path

date_STR = datetime.now().strftime("%y-%m-%d-%H-%M-%S")
logging.basicConfig(level=logging.INFO, format='  [%(levelname)s] %(message)s')
logger = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO, format='  [%(levelname)s] %(message)s')
logger = logging.getLogger(__name__)
from apsimNGpy.core.pythonet_config import is_file_format_modified

IS_NEW_APSIM = is_file_format_modified()
config = configparser.ConfigParser()
config_file = config.read('./unittests/configs.ini')


def send_report(sms, subject, attachment_path=None):
    # Works only on the local machine for the robot to run automatically and send results of what is going on
    # the env vars are set manually on the local machine for security purposes
    all_envs = False
    sender = None
    receiver = None
    key = None
    if os.environ.get('REPORT_TESTS', None):
        sender = os.environ.get('SENDER', sender)
        receiver = os.environ.get('RECEIVER', receiver)
        key = os.environ.get('PASSCODE', key)
        all_envs = all([receiver, sender, key])
    if all_envs:
        from email.mime.base import MIMEBase
        from email.mime.multipart import MIMEMultipart
        from email.mime.text import MIMEText
        # Email credentials
        SMTP_SERVER = "smtp.gmail.com"
        SMTP_PORT = 587
        EMAIL_SENDER = sender
        EMAIL_PASSWORD = key
        EMAIL_RECEIVER = receiver

        # Create the email message
        msg = MIMEMultipart()
        msg["From"] = EMAIL_SENDER
        msg["To"] = EMAIL_RECEIVER
        msg["Subject"] = subject

        # Attach text message
        msg.attach(MIMEText(sms, "plain"))

        # Attach a file if provided
        if attachment_path and os.path.exists(attachment_path):
            with open(attachment_path, "rb") as attachment:
                part = MIMEBase("application", "octet-stream")
                part.set_payload(attachment.read())

            encoders.encode_base64(part)
            part.add_header(
                "Content-Disposition",
                f"attachment; filename={os.path.basename(attachment_path)}",
            )
            msg.attach(part)

        # Send the email
        try:
            with smtplib.SMTP(SMTP_SERVER, SMTP_PORT) as server:
                server.starttls()  # Secure the connection
                server.login(EMAIL_SENDER, EMAIL_PASSWORD)
                server.sendmail(EMAIL_SENDER, EMAIL_RECEIVER, msg.as_string())
                logger.info("Email sent successfully!")
        except Exception as e:
            print("Error:", e)


if IS_NEW_APSIM:
    from apsimNGpy.tests.unittests import test_experiment
from apsimNGpy.tests.unittests import test_model_loader, test_ApsimModel, test_cast_helper, test_CoreModel, \
    test_weathermanager, test_multcores, \
    test_edit_model, test_config, model_tools, test_get_weather_from_web_filename, test_plot_manager

modules = {model_tools,
           test_CoreModel,
           test_ApsimModel,
           test_edit_model,
           test_cast_helper,
           test_model_loader,
           test_config,
           test_weathermanager,
           test_get_weather_from_web_filename,
           test_plot_manager
           }
if IS_NEW_APSIM:
    modules.add(test_experiment)
    modules = (i for i in modules)


def clean_up():
    sc = Path('../scratch')
    for path in sc.rglob("temp_*"):
        try:
            # Ensure file is writable (especially on Windows)
            path.chmod(stat.S_IWRITE)
            path.unlink(missing_ok=True)
        except (PermissionError, FileNotFoundError):
            ...


# Create a test suite combining all the test cases
loader = unittest.TestLoader()
suite = unittest.TestSuite()

for mod in modules:
    suite.addTests(loader.loadTestsFromModule(mod))


def run_multiple_tests():
    load = unittest.TestLoader()
    suit = unittest.TestSuite()
    runner = unittest.TextTestRunner(verbosity=2)
    suit.addTest(load.loadTestsFromModule(test_multcores))
    runner.run(suit)


def run_suite(verbosity_level=2):
    logger.info('Running all tests')
    try:
        runner = unittest.TextTestRunner(verbosity=verbosity_level)
        result = runner.run(suite)

        total_tests = result.testsRun
        num_failures = len(result.failures)
        num_errors = len(result.errors)
        num_passed = total_tests - num_failures - num_errors
        failure_rate = (num_failures + num_errors) / total_tests * 100 if total_tests else 0

        logger.info(f"\n Test Summary:")
        print(f"=====================================")
        logger.info(f"  ✅ Passed  : {num_passed}")
        logger.info(f"  ❌ Failures: {num_failures}")
        logger.info(f"  💥 Errors  : {num_errors}")
        logger.info(f"  📉 Failure Rate: {failure_rate:.2f}%")

        # Create report string
        report = (
            f"Test Suite Summary for {apsim_version()}\n"
            f"====================================\n"
            f"✅ Passed  : {num_passed}\n"
            f"❌ Failures: {num_failures}\n"
            f"💥 Errors  : {num_errors}\n"
            f"📉 Failure Rate: {failure_rate:.2f}%\n"
            f"Date: {date_STR}"
        )

        # Send report

        send_report(sms=report,
                    subject=f"{apsim_version()} Test Suite Report"
                    )

        return report
    finally:
        clean_up()


if __name__ == '__main__':
    # run multi_cores test before
    # run_multiple_tests()
    run_suite(verbosity_level=0)
    from apsimNGpy.core.config import get_bin_use_history



import os.path
import unittest
from pandas import DataFrame
from apsimNGpy.tests.unittests.base_unit_tests import BaseTester
from apsimNGpy.core.experimentmanager import ExperimentManager as Experiment
from pathlib import Path

cwd = Path.cwd()


class TestExperiment(unittest.TestCase):
    def setUp(self):
        self.experiment_path = Path(f"{self._testMethodName}.apsimx")
        self.save_file_name = Path(f"{self._testMethodName}.apsimx")

    def test_add_factor_with_permutes(self):
        t_experiment = Experiment('Maize', out_path=self.experiment_path)
        t_experiment.init_experiment(permutation=True)
        # experiment is initialized in base_tester class SetUp
        t_experiment.add_factor(specification="[Fertilise at sowing].Script.Amount = 0 to 200 step 50",
                                factor_name='Nitrogen')
        t_experiment.run()
        self.assertIsInstance(t_experiment.results, DataFrame)
        if isinstance(t_experiment.results, DataFrame):
            self.assertFalse(t_experiment.results.empty, msg='results are empty')

    def test_add_factor_with_permutes_2_factors(self):
        t_experiment = Experiment('Maize', out_path=self.experiment_path)
        t_experiment.init_experiment(permutation=True)
        # experiment is initialized in base_tester class SetUp
        t_experiment.add_factor(specification="[Fertilise at sowing].Script.Amount = 0 to 200 step 50",
                                factor_name='Nitrogen')
        t_experiment.add_factor(specification="[Sow using a variable rule].Script.Population =4 to 8 step 2",
                                factor_name='Population')
        # Make sure APSIM runs after experiment set up
        t_experiment.run()
        self.assertIsInstance(t_experiment.results, DataFrame)
        if isinstance(t_experiment.results, DataFrame):
            self.assertFalse(t_experiment.results.empty, msg='results are empty')

    def test_inspection(self):

        t_experiment = Experiment('Maize', out_path=self.experiment_path)
        t_experiment.init_experiment(permutation=True)
        # experiment is initialized in base_tester class SetUp
        t_experiment.add_factor(specification="[Fertilise at sowing].Script.Amount = 0 to 200 step 50",
                                factor_name='Nitrogen')
        t_experiment.add_factor(specification="[Sow using a variable rule].Script.Population =4 to 8 step 2",
                                factor_name='Population')
        # Make sure APSIM runs after experiment set up
        t_experiment.run()
        self.assertIsInstance(t_experiment.results, DataFrame)
        if isinstance(t_experiment.results, DataFrame):
            self.assertFalse(t_experiment.results.empty, msg='results are empty')
        managers = all(t_experiment.inspect_model('Manager'))
        self.assertTrue(managers, msg='managers not found in the experiment')

    def test_save(self):
        self.experiment_path.unlink(missing_ok=True)
        self.save_file_name.unlink(missing_ok=True)
        t_experiment = Experiment('Maize', out=os.path.realpath('my_experiment3.apsimx'))
        t_experiment.init_experiment(permutation=True)
        # experiment is initialized in base_tester class SetUp
        t_experiment.add_factor(specification="[Fertilise at sowing].Script.Amount = 0 to 200 step 50",
                                factor_name='Nitrogen')
        t_experiment.add_factor(specification="[Sow using a variable rule].Script.Population =4 to 8 step 2",
                                factor_name='Population')
        filename = str(cwd.joinpath('my_experiment_saved.apsimx'))
        t_experiment.save(file_name=str(self.save_file_name))

        self.assertGreater(self.experiment_path.stat().st_size, 0, msg='experiment not successfully saved')
        self.assertGreater(self.save_file_name.stat().st_size, 0, msg='experiment not successfully saved')

    def test_save_add_factor(self):

        self.save_file_name.unlink(missing_ok=True)
        t_experiment = Experiment('Maize', out_path=self.experiment_path)
        t_experiment.init_experiment(permutation=True)
        # experiment is initialized in base_tester class SetUp
        t_experiment.add_factor(specification="[Fertilise at sowing].Script.Amount = 0 to 200 step 50",
                                factor_name='Nitrogen')
        t_experiment.add_factor(specification="[Sow using a variable rule].Script.Population =4 to 8 step 2",
                                factor_name='Population')

        t_experiment.save(file_name=str(self.save_file_name))
        self.assertGreater(self.save_file_name.stat().st_size, 0, msg='experiment not successfully saved')
        try:
            error = False
            t_experiment.add_factor(specification="[Sow using a variable rule].Script.Population =4 to 8 step 2",
                                    factor_name='Population')
        except Exception as e:
            print(e)
            error = True
        self.assertFalse(error, 'saving and adding factor failed')
        # save again
        self.save_file_name.unlink(missing_ok=True)
        t_experiment.save(file_name=str(self.save_file_name))
        self.assertGreater(self.save_file_name.stat().st_size, 0,
                           msg='experiment not successfully saved after duplicate factor addition test')

    def test_add_factor_with_permutes_fct_rep(self):
        """
        Test that we can handle the unmistakable addition of factors which are identical
        @return:
        """
        t_experiment = Experiment('Maize', out_path=self.experiment_path)
        t_experiment.init_experiment(permutation=True)
        # experiment is initialized in base_tester class SetUp
        t_experiment.add_factor(specification="[Fertilise at sowing].Script.Amount = 0 to 200 step 50",
                                factor_name='Nitrogen')
        t_experiment.add_factor(specification="[Sow using a variable rule].Script.Population =4 to 8 step 2",
                                factor_name='Population')

        t_experiment.add_factor(specification="[Sow using a variable rule].Script.Population =4 to 8 step 2",
                                factor_name='Population')

        self.assertEqual(t_experiment.n_factors, 2)
        # t_experiment.finalize()
        t_experiment.run()
        self.assertIsInstance(t_experiment.results, DataFrame)
        if isinstance(t_experiment.results, DataFrame):
            self.assertFalse(t_experiment.results.empty, msg='results are empty')
        self.assertEqual(t_experiment.n_factors, 2, msg='n_factors is greater than 3  factor repetition tests failed')

    def test_add_factor_without_permutes(self):
        t_experiment = Experiment('Maize', out_path=self.experiment_path)
        t_experiment.init_experiment(permutation=False)
        # experiment is initialized in base_tester class SetUp
        t_experiment.add_factor(specification="[Fertilise at sowing].Script.Amount = 0 to 200 step 50",
                                factor_name='Nitrogen')
        t_experiment.run()
        self.assertIsInstance(t_experiment.results, DataFrame)
        if isinstance(t_experiment.results, DataFrame):
            self.assertFalse(t_experiment.results.empty, msg='results are empty')

    def _clean(self):
        path = Path(self.experiment_path)
        db = path.with_suffix('.db')
        bak = path.with_suffix('.bak')
        db_wal = path.with_suffix('.db-wal')
        db_shm = path.with_suffix('.db-shm')
        db_csv = path.with_suffix('.Report.csv')
        clean_candidates = {bak, db, bak, db_wal, path, db_shm, db_csv}
        for candidate in clean_candidates:
            try:
                candidate.unlink(missing_ok=True)
                print(candidate)
            except PermissionError:
                pass

    def tearDown(self):
        self._clean()


if __name__ == '__main__':
    unittest.main()



from apsimNGpy.optimizer.single import ContinuousVariable, MixedVariable
from apsimNGpy.core.apsim import ApsimModel
from apsimNGpy.settings import  logger
maize_model = ApsimModel("Maize") # replace with the template path

obs = [
    7000.0, 5000.505, 1000.047, 3504.000, 7820.075,
    7000.517, 3587.101, 4000.152, 8379.435, 4000.301
]

class Problem(ContinuousVariable):
    def __init__(self, apsim_model, obs):

        super().__init__(apsim_model=apsim_model)
        self.obs = obs

    def evaluate_objectives(self, **kwargs):
        # This function runs APSIM and compares the predicted maize yield results with observed data.
        predicted = self.apsim_model.run(verbose=False).results.Yield
        # Use root-mean-square error or another metric.
        return self.rmse(self.obs, predicted)

problem = Problem(maize_model, obs)

def maximize_yield(df):
    # Negate yield to convert to a minimization problem
    return -df.Yield.mean()

problem = ContinuousVariable(maize_model, objectives = maximize_yield)

problem.add_control(
    path='.Simulations.Simulation.Field.Fertilise at sowing',
    Amount="?", bounds=[50, 300], v_type='int', start_value=150
)
problem.add_control(
    path='.Simulations.Simulation.Field.Sow using a variable rule',
    Population="?", v_type='int', bounds=[4, 14], start_value=8
)
# test with nelder mead
res_nm = problem.minimize_with_a_local_solver(
    method='Nelder-Mead',
    options={
        'maxiter': 100,
        'disp': True
    }
)
print("Nelder mead results")
logger.info(res_nm)

res_powell = problem.minimize_with_a_local_solver(
    method='Powell',
    options={
        'maxiter': 100,
        'disp': True
    }
)
print("Powell results")
logger.info(res_powell)
# test with powell
# differential evolution
res_de = problem.minimize_with_de(
    popsize=10,
    maxiter=100,
    polish=False  # Set to True if you want to refine with a local solver at the end
)
print("differential evolution results")
logger.info(res_de)


from apsimNGpy.core.apsim import ApsimModel
from apsimNGpy.tests.unittests.base_unit_tests import BaseTester
from pathlib import Path
import matplotlib.pyplot as plt
import unittest

SHOW = False  # use this to turm of interactive plot in the panel


class TestPlotting(BaseTester):

    def setUp(self):
        """
        ApsimModel is defined here
        @return:
        """
        self.test_path = Path(__file__).parent.joinpath(f'test_plots{self._testMethodName}.apsimx')
        self.figure_name = Path(__file__).parent.joinpath(f'figure{self._testMethodName}.png')
        self.db_path = self.test_path.with_suffix('.db')
        self.model = ApsimModel('Maize', out_path=self.test_path)
        self.model.run()

    def test_cat_plot(self):
        'test cat_plot for kind=points, default in seaborn'
        self.model.cat_plot(x="SimulationName", y='Yield')
        fig_enum = plt.get_fignums()
        self.assertTrue(fig_enum)
        self.model.render_plot(save_as=self.figure_name, show=SHOW)
        # check if the figure was successfully saved on file
        self.assertGreater(self.figure_name.stat().st_size, 0, f'Empty figure detected in: {self._testMethodName}')
        # lastly close
        plt.close()

    def test_cat_plot_kind_bar(self):
        """
        test cat_plot for kind=bar
        @return:
        """
        self.model.cat_plot(x="SimulationName", y='Yield', kind='bar', orientation='vertical')
        fig_enum = plt.get_fignums()
        self.assertTrue(fig_enum)
        # labels, saving, and showing plots are aggregated in render_plots
        self.model.render_plot(save_as=self.figure_name, show=SHOW)
        # check if a figure was successfully saved on file
        self.assertGreater(self.figure_name.stat().st_size, 0, f'Empty figure detected in: {self._testMethodName}')
        # lastly close
        plt.close()

    def test_cat_plot_kind_box(self):
        """
        test cat_plot for kind=box
        @return:
        """
        self.model.cat_plot(x="SimulationName", y='Yield', kind='box', orientation='vertical')
        fig_enum = plt.get_fignums()
        self.assertTrue(fig_enum)
        self.model.render_plot(save_as=self.figure_name, show=SHOW)
        # check if figure was successfully saved on file
        self.assertGreater(self.figure_name.stat().st_size, 0, f'Empty figure detected in: {self._testMethodName}')
        # lastly close
        plt.close()

    def test_relplot_kind_line(self):
        """
        test relative plot for kind=line
        @return:
        """
        self.model.relplot(x='Maize.Grain.Wt', y='Yield', kind='line')
        fig_enum = plt.get_fignums()
        self.assertTrue(fig_enum)
        self.model.render_plot(save_as=self.figure_name, show=SHOW)
        # check if figure was successfully saved on file
        self.assertGreater(self.figure_name.stat().st_size, 0, f'Empty figure detected in: {self._testMethodName}')
        # lastly close
        plt.close()

    def test_line_plot(self):
        """test line_plot"""
        self.figure_name.unlink(missing_ok=True)
        self.model.series_plot(y='Yield', x='Clock.Today')
        fig_enum = plt.get_fignums()
        self.assertTrue(fig_enum)
        # render before fig evaluation test because render closes the plot
        self.model.render_plot(save_as=self.figure_name, show=SHOW)
        # check if figure was successfully saved on file
        self.assertGreater(self.figure_name.stat().st_size, 0, f'Empty figure detected in: {self._testMethodName}')
        plt.close()

    def test_distribution_plot(self):
        """
        test distribution_plot
        @return:
        """

        self.figure_name.unlink(missing_ok=True)
        self.model.distribution("Yield")
        fig_enum = plt.get_fignums()
        self.assertTrue(fig_enum)
        # labels, saving, and showing plots are aggregated in render_plots
        self.model.render_plot(save_as=self.figure_name, show=SHOW)
        # check if figure was successfully saved on file
        self.assertGreater(self.figure_name.stat().st_size, 0, f'Empty figure detected in: {self._testMethodName}')
        plt.close()

    def test_scatter_plot(self):
        self.figure_name.unlink(missing_ok=True)
        self.model.scatter_plot(x='Maize.Grain.Wt', y='Yield')
        fig_enum = plt.get_fignums()
        self.assertTrue(fig_enum)
        self.model.render_plot(save_as=self.figure_name, show=SHOW)
        # check if figure was successfully saved on file
        self.assertGreater(self.figure_name.stat().st_size, 0, f'Empty figure detected in: {self._testMethodName}')
        plt.close()

    def test_scatter_plot_with_external_df(self):
        self.figure_name.unlink(missing_ok=True)
        df = self.model.results
        self.model.scatter_plot(x='Maize.Grain.Wt', y='Yield', data=df)
        fig_enum = plt.get_fignums()
        self.assertTrue(fig_enum)
        self.model.render_plot(save_as=self.figure_name, show=SHOW)
        self.assertGreater(self.figure_name.stat().st_size, 0, f'Empty figure detected in: {self._testMethodName}')
        plt.close()

    def tearDown(self):
        self.test_path.unlink(missing_ok=True)
        self.figure_name.unlink(missing_ok=True)
        self.db_path.unlink(missing_ok=True)
        # cleans up everything related to the model stored on the computer
        self.model.clean_up(db=True)


if __name__ == '__main__':
    unittest.main()



from pathlib import Path
from apsimNGpy.core.apsim import ApsimModel
from apsimNGpy.core_utils.database_utils import read_db_table
from apsimNGpy.parallel.process import custom_parallel
import pandas as pd
from sqlalchemy import create_engine


DATABAse = str(Path('test_custom.db').resolve())

# define function to insert insert results
def insert_results(db_path, results, table_name):
    """
    Insert a pandas DataFrame into a SQLite table.

    Parameters
    ----------
    db_path : str or Path
        Path to the SQLite database file.
    results : pandas.DataFrame
        DataFrame to insert into the database.
    table_name : str
        Name of the table to insert the data into.
    """
    if not isinstance(results, pd.DataFrame):
        raise TypeError("`results` must be a pandas DataFrame")

    engine = create_engine(f"sqlite:///{db_path}")
    results.to_sql(table_name, con=engine, if_exists='append', index=False)


def worker(nitrogen_rate, model):
    out_path = Path(f"_{nitrogen_rate}.apsimx").resolve()
    model = ApsimModel(model, out_path=out_path)
    model.edit_model("Models.Manager", model_name='Fertilise at sowing', Amount=nitrogen_rate)
    model.run(report_name="Report")
    df = model.results
    # we can even create column for each simulation
    df['nitrogen rate'] = nitrogen_rate
    insert_results(DATABAse, df, 'Report')
    model.clean_up()


if __name__ == '__main__':

    for _ in custom_parallel(worker, range(0, 400, 10), 'Maize', n_cores=6, use_threads=False):
        pass
    # get the results
    data = read_db_table(DATABAse, report_name="Report")



from apsimNGpy.core.apsim import ApsimModel
from apsimNGpy.core.model_tools import Models, ModelTools


class CompareCultivar(ApsimModel):
    def __init__(self, model, out_path=None):
        super().__init__(model=model, out_path=out_path)
        self.base_simulations = None

    def refresh_base_simulations(self):
        if self.base_simulations is None:
            base_sim = self.Simulations.FindInScope[Models.Core.Simulation]()
            base_sim = ModelTools.CLONER(base_sim)
            self.base_simulations = base_sim
        return self.base_simulations

    def simulate_different_locations(self, locations: list[tuple]):
        sim = self.refresh_base_simulations()

        sims = self.Simulations.FindAllInScope[Models.Core.Simulation]()
        for s in sims:
            ModelTools.DELETE(s)
        for lonlat in locations:
            sim_name = f"sim_{lonlat[0]}_{lonlat[1]}"
            clone_sim = ModelTools.CLONER(sim)
            print(clone_sim)
            clone_sim.Name = sim_name
            ModelTools.ADD(clone_sim, self.Simulations)

    def simulate_different_cultivars(self, cultivar_names):
        cultivars = self.inspect_model('Cultivar', fullpath=False)
        lower = dict(zip([i.lower() for i in cultivars], cultivars))

        return lower

    model = ApsimModel('Maize')
    model.add_factor(specification="[Sow using a variable rule].Script.CultivarName = B_110, A_90")


if __name__ == '__main__':
    # model = CompareCultivar(model='Maize')
    import pandas as pd

    mod = CompareCultivar('Maize')
    lonlat_iowa = [
        (-96.4003, 42.5006),  # Sioux City (NW)
        (-94.1680, 42.0347),  # Fort Dodge (N Central)
        (-96.7870, 41.2619),  # Council Bluffs (SW)
        (-93.6208, 41.5868),  # Des Moines (Central)
        (-93.0977, 41.0239),  # Osceola (South Central)
        (-91.5302, 41.6611),  # Iowa City (SE)
        (-91.1800, 42.5006),  # Dubuque (NE)
        (-94.7403, 41.3303),  # Atlantic (SW)
        (-92.9108, 42.0354),  # Marshalltown (Central East)
        (-92.4083, 41.2960),  # Ottumwa (SE)
        (-90.5736, 41.5236),  # Davenport (far East)
        (-92.4499, 41.5772),  # Oskaloosa (SE Central)
        (-95.8547, 41.7370),  # Harlan (West Central)
        (-93.7680, 43.3841),  # Northwood (North)
        (-94.1564, 43.0731),  # Estherville (Northwest)
        (-91.1129, 40.8075),  # Keokuk (far SE)
        (-95.0289, 42.0350),  # Carroll (Central West)
        (-93.9179, 42.4975),  # Humboldt (North Central)
        (-93.6020, 40.7398),  # Lamoni (South Central)
        (-91.6710, 41.2417),  # Washington (SE)
    ]
    mod.simulate_different_locations(lonlat_iowa)

    pd.set_option('display.max_columns', None)

    # model = ApsimModel('Maize')
    # model.create_experiment(permutation=False)
    # model.add_factor(specification="[Sow using a variable rule].Script.CultivarName =  Dekalb_XL82, Melkassa, Pioneer_34K77, Laila, B_110, A_90")
    # model.run()
    # df = model.results
    # from matplotlib import pyplot as plt
    #
    # ax = df.boxplot(column='Yield', by='CultivarName',figsize=(10,8), grid=False)
    #
    # # Customize the plot
    # plt.title('Maize yield boxplot grouped by Cultivar', fontsize=20)
    # plt.suptitle('')  # Remove the default automatic title
    # plt.xlabel('Cultivar Name', fontsize=20)
    # plt.ylabel('Maize Yield (kg ha$^{-1}$)',fontsize=20 )
    #
    # plt.savefig(r'D:\PACKAGES\apsimNGpy-documentations\images/Cultivar_Maize.png', dpi=600)
    #
    # import seaborn as sns
    # import matplotlib.pyplot as plt
    #
    # plt.figure(figsize=(10, 6))
    #
    # # Lineplot grouped by CultivarName
    # df['Year'] = [date.year for date in pd.to_datetime(df['Clock.Today'])]
    # sns.lineplot(data=df, x='Year', y='Yield', hue='CultivarName', marker='o')
    #
    # # Labels and formatting
    # plt.title('Maize Yield Over Years by Cultivar', fontsize=16)
    # plt.xlabel('Year', fontsize=14)
    # plt.ylabel('Maize Yield (kg ha$^{-1}$)', fontsize=14)
    # plt.legend(title='Cultivar', bbox_to_anchor=(1.05, 1), loc='upper left')
    # plt.tight_layout()
    #
    # # Save before showing
    # plt.savefig(r'./Cultivar_Maize_Yield_Lineplot.png', dpi=600)
    # plt.show()
    #



from pathlib import Path
import logging
from typing import Union

logging.basicConfig(
    level=logging.INFO,  # or DEBUG, WARNING, ERROR, CRITICAL
    format='\n[%(levelname)s]: %(message)s'
)


def clean(root_dir: Union[str, Path], recursive=False) -> None:
    path = Path(root_dir).resolve()
    glob = 'rglob' if recursive else 'glob'

    def _get_files():
        ax = getattr(path, glob)('*.apsimx')
        db = getattr(path, glob)('*.db')
        remove = list(ax) + list(db) + list(getattr(path, glob)('*.db-shm')) + list(getattr(path, glob)('*.csv'))
        return remove

    fl = _get_files()
    len1 = len(fl)

    for i in fl:
        try:
            i.unlink(missing_ok=True)
        except PermissionError:
            logging.info(f"skipping '{i}' due to permission error")
    # get files again
    fl2 = _get_files()
    len2 = len(fl2)
    removed = len1 - len2
    logging.info(f" Cleaned {removed} unwanted files")


if __name__ == '__main__':
    clean('.')



from apsimNGpy.optimizer.moo import MultiObjectiveProblem, compute_hyper_volume, NSGA2
from pymoo.optimize import minimize
import matplotlib.pyplot as plt
from apsimNGpy.core.apsim import ApsimModel as Runner
import numpy as np

# 1. Setup APSIM model
runner = Runner("Maize")
runner.add_report_variable('[Soil].Nutrient.NO3.kgha[1] as nitrate', report_name='Report')

# below enables defining other parameters that will be required for editing
_vars = [
    {'path': '.Simulations.Simulation.Field.Fertilise at sowing', 'Amount': "?", "bounds": [50, 300],
     "v_type": "float"},
    {'path': '.Simulations.Simulation.Field.Sow using a variable rule', 'Population': "?", 'v_type': 'float',
     'bounds': [4, 14]}
]


# 3. Define objective functions
def maximize_yield(df):
    return -df['Yield'].mean()


def nitrate_leaching(df):
    return df['nitrate'].sum()


objectives = [maximize_yield, nitrate_leaching]


# prob = MultiObjectiveProblem(runner, _vars, objectives)

def run_example():
    # 4. Define problem and optimizer
    problem = MultiObjectiveProblem(runner, objectives=objectives, decision_vars=_vars)
    # problem.add_control(
    #     **{'path': '.Simulations.Simulation.Field.Fertilise at sowing', 'Amount': "?", "bounds": [50, 300],
    #        "v_type": "float"})
    # problem.add_control(
    #     **{'path': '.Simulations.Simulation.Field.Sow using a variable rule', 'Population': "?", 'v_type': 'float',
    #        'bounds': [4, 14]})
    algorithm = NSGA2(pop_size=20, )

    # 5. Optimize
    result = minimize(problem.get_problem(),
                      algorithm,
                      ('n_gen', 2),
                      seed=1,
                      verbose=True)

    # 6. View Pareto front
    import matplotlib.pyplot as plt

    F = result.F
    plt.scatter(F[:, 0] * -1, F[:, 1])
    plt.xlabel("Yield")
    plt.ylabel("N Leaching")
    plt.title("Trade-offs between yield and nitrate leaching")
    plt.show()

    hiv = compute_hyper_volume(F, normalize=True, )
    print(hiv)
    return result


if __name__ == '__main__':
    res = run_example()
    import os

    from sklearn.preprocessing import MinMaxScaler

    scaler = MinMaxScaler()
    f_scaled = scaler.fit_transform(res.F)

    import numpy as np
    from pymoo.visualization.scatter import Scatter
    from pymoo.mcdm.high_tradeoff import HighTradeoffPoints

    pf = res.F
    dm = HighTradeoffPoints()

    I = dm(f_scaled)
    tdp = f_scaled[I]

    plt.scatter(f_scaled[:, 0], f_scaled[:, 1])
    plt.scatter(tdp[0][0] * -1, tdp[0][0], c="red", s=100, label='Selected Solution')
    plt.xlabel("Yield")
    plt.ylabel("N Leaching")
    plt.title("Pareto Front")
    plt.show()

    weights = 1 / np.array([0.6, 0.4])


    def compromise_with_weights(res, weights):
        assert np.sum(weights) == 1, "weights must sum to 1"
        from pymoo.decomposition.asf import ASF
        scaler = MinMaxScaler()
        decomp = ASF()
        nF = scaler.fit_transform(res.F)
        i = decomp.do(nF, 1 / weights).argmin()
        # plot
        plt.scatter(res.F[:, 0], res.F[:, 1], c='blue')
        plt.scatter(res.F[i, 0], res.F[i, 1], c="red", s=100, label='Selected Solution')
        plt.xlabel("Yield")
        plt.ylabel("N Leaching")
        plt.title("Pareto Front")
        plt.show()
        return res.X[i, 0], res.X[i, 1]



import os.path
import unittest

from apsimNGpy.tests.unittests.base_unit_tests import BaseTester

from apsimNGpy.manager import weathermanager

LONLAT = 35.582520, 0.347596  # location is kampala
AMES_LONLAT = -93.0, 42.03534

AMES_LONLAT2 = -92.0, 42.03534


class TestWeatherManager(BaseTester):
    def setUp(self):
        self.start = 2000
        self.end = 2001
        self.LONLAT = 35.582520, 0.347596
        self.weather_file_name = f"{self._testMethodName}_{self.start}_{self.end}.met"

    def test_get_weather(self):
        if os.path.isfile(self.weather_file_name):
            os.remove(self.weather_file_name)
        met = weathermanager.get_weather(LONLAT, start=self.start, end=self.end,
                                         source='nasa', filename=self.weather_file_name, )
        self.assertIsInstance(met, str)
        self.assertTrue(os.path.exists(met))
        met = weathermanager.get_weather(AMES_LONLAT, source='daymet')
        self.assertIsInstance(met, str)
        self.assertTrue(os.path.exists(met))

    def test_is_within_usa_mainland(self):
        type_bool = weathermanager._is_within_USA_mainland(self.LONLAT)
        self.assertIsInstance(type_bool, bool, f"{self._testMethodName} did not run successfully")

    def test_read_met(self):
        wf = weathermanager.get_weather(AMES_LONLAT2, start=self.start, end=self.end, source='daymet',
                                        filename=self.weather_file_name)
        if wf is not None:
            df = weathermanager.read_apsim_met(wf)
            if df is not None:
                self.assertFalse(df.empty, 'read_apsim_met failed')
                # test writing to file
                write_file = weathermanager.write_edited_met(wf, df, filename=self.weather_file_name)
                self.assertTrue(write_file)
                exi_file = os.path.isfile(write_file)
                self.assertTrue(exi_file, msg=f"Writing met to file failed")

    def tearDown(self):
        if os.path.isfile(self.weather_file_name):
            os.remove(self.weather_file_name)


if __name__ == '__main__':
    unittest.main()
